#!/usr/bin/env python3

'''Unix domain socket connection and manager.'''

import asyncio
from typing import Tuple

from . import Connection
from . import ConnectionManager
from ..exceptions import AIOSpamcConnectionFailed


class UnixConnectionManager(ConnectionManager):
    '''Creates new connections based on Unix domain socket path provided.'''

    def __init__(self, path: str, loop: asyncio.AbstractEventLoop = None) -> None:
        '''Constructor for UnixConnectionManager.

        :param path: Path of the socket.
        :param loop: The asyncio event loop.
        '''

        self.path = path
        super().__init__(loop)

    def __repr__(self) -> str:
        return 'UnixConnectionManager(path={})'.format(repr(self.path))

    def new_connection(self) -> 'UnixConnection':
        '''Creates a new Unix domain socket connection.

        :raises AIOSpamcConnectionFailed:
        '''

        return UnixConnection(self.path, self.loop)


class UnixConnection(Connection):
    '''Manages a Unix domain socket connection.'''

    def __init__(self, path: str, loop: asyncio.AbstractEventLoop = None) -> None:
        '''Constructor for UnixConnection.

        :param path: Path of the socket.
        :param loop: The asyncio event loop.
        '''

        self.path = path
        super().__init__(loop)

    def __repr__(self) -> str:
        return 'UnixConnection(path={})'.format(repr(self.path))

    async def open(self) -> Tuple[asyncio.StreamReader, asyncio.StreamWriter]:
        '''Opens a connection.

        :raises AIOSpamcConnectionFailed:
        '''

        try:
            reader, writer = await asyncio.open_unix_connection(self.path,
                                                                loop=self.loop)
        except (ConnectionRefusedError, OSError) as error:
            raised = AIOSpamcConnectionFailed(error)
            self.logger.exception('Exception occurred when connecting: %s', raised)
            raise raised

        return reader, writer

    @property
    def connection_string(self) -> str:
        '''String representation of the connection.'''

        return self.path



#!/usr/bin/env python3

'''Collection of request and response header value objects.'''

import getpass

from .options import ActionOption, MessageClassOption


class HeaderValue:
    pass


class GenericHeaderValue(HeaderValue):
    '''Generic header value.'''
    
    def __init__(self, value: str, encoding='utf8') -> None:
        self.value = value
        self.encoding = encoding

    def __str__(self):
        return 'value={}, encoding={}'.format(repr(self.value), repr(self.encoding))

    def __bytes__(self) -> bytes:
        return self.value.encode(self.encoding)

    def __eq__(self, other):
        try:
            return other.value == self.value and other.encoding == self.encoding
        except AttributeError:
            return False

    def __repr__(self):
        return '{}(value={}, encoding={})'.format(
            self.__class__.__qualname__,
            repr(self.value),
            repr(self.encoding)
        )


class CompressValue(HeaderValue):
    '''Compress header.  Specifies what encryption scheme to use.  So far only
    'zlib' is supported.
    '''

    def __init__(self, algorithm='zlib') -> None:
        '''Constructor

        :param algorithm: Compression algorithm to use.  Currently on zlib is supported.
        '''

        self.algorithm = algorithm

    def __str__(self):
        return 'algorithm={}'.format(repr(self.algorithm))

    def __eq__(self, other):
        try:
            return self.algorithm == other.algorithm
        except AttributeError:
            return False

    def __repr__(self) -> str:
        return '{}()'.format(self.__class__.__qualname__)

    def __bytes__(self) -> bytes:
        return self.algorithm.encode('ascii')


class ContentLengthValue(HeaderValue):
    '''ContentLength header.  Indicates the length of the body in bytes.'''

    def __init__(self, length: int = 0) -> None:
        '''ContentLength constructor.

        :param length: Length of the body.
        '''
        self.length = length

    def __str__(self):
        return 'length={}'.format(repr(self.length))

    def __eq__(self, other):
        try:
            return self.length == other.length
        except AttributeError:
            return False

    def __bytes__(self) -> bytes:
        return str(self.length).encode('ascii')

    def __repr__(self) -> str:
        return '{}(length={})'.format(self.__class__.__qualname__, self.length)


class MessageClassValue(HeaderValue):
    '''MessageClass header.  Used to specify whether a message is 'spam' or
    'ham.'
    '''

    def __init__(self, value: MessageClassOption = None) -> None:
        '''MessageClass constructor.

        :param value: Specifies the classification of the message.
        '''

        self.value = value or MessageClassOption.ham

    def __str__(self):
        return self.value.name

    def __eq__(self, other):
        try:
            return self.value == other.value
        except AttributeError:
            return False

    def __bytes__(self) -> bytes:
        return self.value.name.encode('ascii')

    def __repr__(self) -> str:
        return '{}(value={})'.format(self.__class__.__qualname__, str(self.value))


class SetOrRemoveValue(HeaderValue):
    '''Base class for headers that implement "local" and "remote" rules.'''

    def __init__(self, action: ActionOption = None) -> None:
        '''_SetRemoveBase constructor.

        :param action: Actions to be done on local or remote.
        '''

        self.action = action or ActionOption(local=False, remote=False)

    def __str__(self):
        return 'local={}, remote={}'.format(str(self.action.local), str(self.action.remote))

    def __eq__(self, other):
        try:
            return self.action == other.action
        except AttributeError:
            return False

    def __bytes__(self) -> bytes:
        if not self.action.local and not self.action.remote:
            # if nothing is set, then return a blank string so the request
            # doesn't get tainted
            return b''

        values = []
        if self.action.local:
            values.append(b'local')
        if self.action.remote:
            values.append(b'remote')

        return b', '.join(values)

    def __repr__(self) -> str:
        return '{}(action={})'.format(self.__class__.__qualname__,
                                      repr(self.action))


class SpamValue(HeaderValue):
    '''Spam header.  Used by the SPAMD service to report on if the submitted
    message was spam and the score/threshold that it used.'''

    def __init__(self, value: bool = False, score: float = 0.0, threshold: float = 0.0) -> None:
        '''Spam header constructor.

        :param value: True if the message is spam, False if not.
        :param score: Score of the message after being scanned.
        :param threshold: Threshold of which the message would have been marked as spam.
        '''

        self.value = value
        self.score = score
        self.threshold = threshold

    def __str__(self):
        return 'value={}, score={}, threshold={}'.format(str(self.value), str(self.score), str(self.threshold))

    def __eq__(self, other):
        try:
            return all([self.value == other.value, self.score == other.score, self.threshold == other.threshold])
        except AttributeError:
            return False

    def __bytes__(self) -> bytes:
        return b'%b ; %.1f / %.1f' % (str(self.value).encode('ascii'),
                                          self.score,
                                          self.threshold)

    def __repr__(self) -> str:
        return '{}(value={}, score={}, threshold={})'.format(self.__class__.__qualname__,
                                                             self.value,
                                                             self.score,
                                                             self.threshold)


class UserValue(HeaderValue):
    '''User header.  Used to specify which user the SPAMD service should use
    when loading configuration files.'''

    def __init__(self, name: str = None) -> None:
        '''User constructor.

        :param name: Name of the user account.
        '''

        self.name = name or getpass.getuser()

    def __str__(self):
        return 'name={}'.format(self.name)

    def __eq__(self, other):
        try:
            return self.name == other.name
        except AttributeError:
            return False

    def __bytes__(self) -> bytes:
        return self.name.encode('ascii')

    def __repr__(self) -> str:
        return '{}(name={})'.format(self.__class__.__qualname__, repr(self.name))



#!/usr/bin/env python3

'''Data structures used for function parameters.'''

from collections import namedtuple
from enum import IntEnum


class MessageClassOption(IntEnum):
    '''Option to be used for the MessageClass header.'''

    spam = 1
    ham = 2


ActionOption = namedtuple('ActionOption', ['local', 'remote'])
'''Option to be used in the DidRemove, DidSet, Set, and Remove headers.

local : bool
    An action will be performed on the SPAMD service's local database.
remote : bool
    An action will be performed on the SPAMD service's remote database.
'''



#!/usr/bin/env python3

'''Contains classes used for responses.'''

import enum
from typing import Mapping, SupportsBytes, Union
import zlib

from .common import SpamcBody, SpamcHeaders
from .exceptions import (UsageException, DataErrorException, NoInputException, NoUserException,
                         NoHostException, UnavailableException, InternalSoftwareException, OSErrorException,
                         OSFileException, CantCreateException, IOErrorException, TemporaryFailureException,
                         ProtocolException, NoPermissionException, ConfigException, TimeoutException, ResponseException)
from .header_values import ContentLengthValue, HeaderValue


class Status(enum.IntEnum):
    '''Enumeration of status codes that the SPAMD will accompany with a
    response.

    Reference: https://svn.apache.org/repos/asf/spamassassin/trunk/spamd/spamd.raw
    Look for the %resphash variable.
    '''

    def __new__(cls, value, exception=None, description=''):
        obj = int.__new__(cls, value)
        obj._value_ = value
        obj.exception = exception
        obj.description = description

        return obj

    EX_OK = 0, None, 'No problems'
    EX_USAGE = 64, UsageException, 'Command line usage error'
    EX_DATAERR = 65, DataErrorException, 'Data format error'
    EX_NOINPUT = 66, NoInputException, 'Cannot open input'
    EX_NOUSER = 67, NoUserException, 'Addressee unknown'
    EX_NOHOST = 68, NoHostException, 'Host name unknown'
    EX_UNAVAILABLE = 69, UnavailableException, 'Service unavailable'
    EX_SOFTWARE = 70, InternalSoftwareException, 'Internal software error'
    EX_OSERR = 71, OSErrorException, 'System error (e.g., can\'t fork)'
    EX_OSFILE = 72, OSFileException, 'Critical OS file missing'
    EX_CANTCREAT = 73, CantCreateException, 'Can\'t create (user) output file'
    EX_IOERR = 74, IOErrorException, 'Input/output error'
    EX_TEMPFAIL = 75, TemporaryFailureException, 'Temp failure; user is invited to retry'
    EX_PROTOCOL = 76, ProtocolException, 'Remote error in protocol'
    EX_NOPERM = 77, NoPermissionException, 'Permission denied'
    EX_CONFIG = 78, ConfigException, 'Configuration error'
    EX_TIMEOUT = 79, TimeoutException, 'Read timeout'


class Response:
    '''Class to encapsulate response.'''

    def __init__(
            self,
            version: str = '1.5',
            status_code: Union[Status, int] = 0,
            message: str = '',
            headers: Mapping[str, HeaderValue] = None,
            body: bytes = b'',
            **_
    ):
        '''Response constructor.

        :param version: Version reported by the SPAMD service response.
        :param status_code: Success or error code.
        :param message: Message associated with status code.
        :param body: Byte string representation of the body.
        :param headers: Collection of headers to be added.
        '''

        self.version = version

        self.headers = SpamcHeaders(headers=headers)

        try:
            self.status_code = Status(status_code)
        except ValueError:
            self.status_code = status_code

        self.message = message

        self.body = body

    def __bytes__(self) -> bytes:
        if 'Compress' in self.headers:
            body = zlib.compress(self.body)
        else:
            body = self.body

        if len(body) > 0:
            self.headers['Content-length'] = ContentLengthValue(length=len(body))

        if isinstance(self.status_code, Status):
            status = self.status_code.value
            message = str(self.status_code).encode('ascii')
        else:
            status = self.status_code
            message = self.message.encode('ascii')

        return b'SPAMD/%(version)b ' \
               b'%(status)d ' \
               b'%(message)b\r\n' \
               b'%(headers)b\r\n' \
               b'%(body)b' % {b'version': self.version.encode('ascii'),
                              b'status': status,
                              b'message': message,
                              b'headers': bytes(self.headers),
                              b'body': body}

    def __str__(self):
        return '<{} - {}: {} object at {}>'.format(
            self.status_code.value,
            self.status_code.name,
            '.'.join([self.__class__.__module__, self.__class__.__qualname__]),
            id(self)
        )

    body = SpamcBody()  # type: Union[bytes, SupportsBytes]

    def raise_for_status(self) -> None:
        if isinstance(self.status_code, Status):
            if self.status_code.exception:
                raise self.status_code.exception
            return
        else:
            raise ResponseException



#!/usr/bin/env python

'''Connection and ConnectionManager base classes.'''

import asyncio
import logging
from typing import Tuple, Union, SupportsBytes


class Connection:
    '''Base class for connection objects.'''

    def __init__(self, loop: asyncio.AbstractEventLoop = None):
        '''Connection constructor.

        :param loop: The asyncio event loop.
        '''

        self.connected = False
        self.loop = loop or asyncio.get_event_loop()
        self.logger = logging.getLogger(__name__)

    async def __aenter__(self):
        self.logger.debug('Connecting to %s', self.connection_string)
        self.reader, self.writer = await self.open()
        self.connected = True
        self.logger.debug('Connected to %s', self.connection_string)
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self.logger.debug('Closing connection to %s', self.connection_string)
        self.close()
        self.logger.debug('Closed connection to %s', self.connection_string)

    async def open(self) -> Tuple[asyncio.StreamReader, asyncio.StreamWriter]:
        '''Connect to a service.

        :raises AIOSpamcConnectionFailed:
        '''

        raise NotImplementedError

    @property
    def connection_string(self) -> str:
        '''String representation of the connection.'''

        raise NotImplementedError

    def close(self) -> None:
        '''Closes the connection.'''

        self.writer.close()
        self.connected = False
        del self.reader, self.writer

    async def send(self, data: Union[bytes, SupportsBytes]) -> None:
        '''Sends data through the connection.'''

        self.writer.write(data)
        await self.writer.drain()

    async def receive(self) -> bytes:
        '''Receives data from the connection.'''

        return await self.reader.read()


class ConnectionManager:
    '''Stores connection parameters and creates connections.'''

    def __init__(self, loop: asyncio.AbstractEventLoop = None):
        self.loop = loop or asyncio.get_event_loop()

    def new_connection(self) -> Connection:
        '''Creates a connection object.'''

        raise NotImplementedError



#!/usr/bin/env python3

'''Collection of exceptions.'''


# Exceptions raised from the Client object
class ClientException(Exception):
    '''Base class for exceptions raised from the client.'''
    pass


class BadRequest(ClientException):
    '''Request is not in the expected format.'''
    pass


class BadResponse(ClientException):
    '''Response is not in the expected format.'''
    pass


# ConnectionManager and Connection object exceptions
class AIOSpamcConnectionException(Exception):
    '''Base class for exceptions from the connection.'''
    pass


class AIOSpamcConnectionFailed(AIOSpamcConnectionException):
    '''Connection failed.'''
    pass


class ResponseException(Exception):
    '''Base class for exceptions raised from a response.'''
    pass


class UsageException(ResponseException):
    '''Command line usage error.'''
    code = 64


class DataErrorException(ResponseException):
    '''Data format error.'''
    code = 65


class NoInputException(ResponseException):
    '''Cannot open input.'''
    code = 66


class NoUserException(ResponseException):
    '''Addressee unknown.'''
    code = 67


class NoHostException(ResponseException):
    '''Hostname unknown.'''
    code = 68


class UnavailableException(ResponseException):
    '''Service unavailable.'''
    code = 69


class InternalSoftwareException(ResponseException):
    '''Internal software error.'''
    code = 70


class OSErrorException(ResponseException):
    '''System error (e.g. can't fork the process).'''
    code = 71


class OSFileException(ResponseException):
    '''Critical operating system file missing.'''
    code = 72


class CantCreateException(ResponseException):
    '''Can't create (user) output file.'''
    code = 73


class IOErrorException(ResponseException):
    '''Input/output error.'''
    code = 74


class TemporaryFailureException(ResponseException):
    '''Temporary failure, user is invited to try again.'''
    code = 75


class ProtocolException(ResponseException):
    '''Remote error in protocol.'''
    code = 76


class NoPermissionException(ResponseException):
    '''Permission denied.'''
    code = 77


class ConfigException(ResponseException):
    '''Configuration error.'''
    code = 78


class TimeoutException(ResponseException):
    '''Read timeout.'''
    code = 79



#!/usr/bin/env python3

'''Contains all requests that can be made to the SPAMD service.'''

from typing import Mapping, SupportsBytes, Union
import zlib

from .common import SpamcBody, SpamcHeaders
from .header_values import ContentLengthValue, HeaderValue


class Request:
    '''SPAMC request object.'''

    def __init__(
        self,
        verb: str = None,
        version: str = '1.5',
        headers: Mapping[str, Union[str, HeaderValue]] = None,
        body: Union[bytes, SupportsBytes] = b'',
        **_
    ) -> None:
        '''Request constructor.

        :param verb: Method name of the request.
        :param version: Version of the protocol.
        :param headers: Collection of headers to be added.
        :param body: Byte string representation of the body.
        '''

        self.verb = verb
        self.version = version
        self.headers = SpamcHeaders(headers=headers)
        self.body = body

    def __bytes__(self) -> bytes:
        if 'Compress' in self.headers.keys():
            body = zlib.compress(self.body)
        else:
            body = self.body

        if len(body) > 0:
            self.headers['Content-length'] = ContentLengthValue(length=len(body))

        request = (b'%(verb)b '
                   b'SPAMC/%(version)b'
                   b'\r\n'
                   b'%(headers)b\r\n'
                   b'%(body)b')

        return request % {b'verb': self.verb.encode('ascii'),
                          b'version': self.version.encode('ascii'),
                          b'headers': bytes(self.headers),
                          b'body': body}

    def __str__(self):
        return '<{}: {} object at {}>'.format(
            self.verb,
            '.'.join([self.__class__.__module__, self.__class__.__qualname__]),
            id(self)
        )

    body = SpamcBody()  # type: Union[bytes, SupportsBytes]



#!/usr/bin/env python3

'''aiospamc package.

An asyncio-based library to communicate with SpamAssassin's SPAMD service.'''

import asyncio
from typing import SupportsBytes, Union

from .client import Client
from .options import ActionOption, MessageClassOption
from .responses import Response


__all__ = ('Client',
           'MessageClassOption',
           'ActionOption')

__author__ = 'Michael Caley'
__copyright__ = 'Copyright 2016-2019 Michael Caley'
__license__ = 'MIT'
__version__ = '0.6.0'
__email__ = 'mjcaley@darkarctic.com'


async def check(
        message: Union[bytes, SupportsBytes],
        *,
        host: str = 'Localhost',
        port: int = 783,
        loop: asyncio.AbstractEventLoop = None,
        **kwargs) -> Response:
    '''Checks a message if it's spam and return a response with a score header.

    :param message: Copy of the message.
    :param host: Hostname or IP address of the SPAMD service, defaults to localhost.
    :param port: Port number for the SPAMD service, defaults to 783.
    :param loop: The asyncio event loop.
    :param kwargs:
        Additional options to pass to the :class:`aiospamc.client.Client` instance.

    :return:
        A successful response with a "Spam" header showing if the message is
        considered spam as well as the score.

    :raises BadResponse: If the response from SPAMD is ill-formed this exception will be raised.
    :raises AIOSpamcConnectionFailed: Raised if an error occurred when trying to connect.
    :raises UsageException: Error in command line usage.
    :raises DataErrorException: Error with data format.
    :raises NoInputException: Cannot open input.
    :raises NoUserException: Addressee unknown.
    :raises NoHostException: Hostname unknown.
    :raises UnavailableException: Service unavailable.
    :raises InternalSoftwareException: Internal software error.
    :raises OSErrorException: System error.
    :raises OSFileException: Operating system file missing.
    :raises CantCreateException: Cannot create output file.
    :raises IOErrorException: Input/output error.
    :raises TemporaryFailureException: Temporary failure, may reattempt.
    :raises ProtocolException: Error in the protocol.
    :raises NoPermissionException: Permission denied.
    :raises ConfigException: Error in configuration.
    :raises TimeoutException: Timeout during connection.
    '''

    c = Client(host=host, port=port, **kwargs, loop=loop)
    response = await c.check(message)

    return response


async def headers(
        message: Union[bytes, SupportsBytes],
        *,
        host: str = 'Localhost',
        port: int = 783,
        loop: asyncio.AbstractEventLoop = None,
        **kwargs) -> Response:
    '''Checks a message if it's spam and return the modified message headers.

    :param message: Copy of the message.
    :param host: Hostname or IP address of the SPAMD service, defaults to localhost.
    :param port: Port number for the SPAMD service, defaults to 783.
    :param loop: The asyncio event loop.
    :param kwargs:
        Additional options to pass to the :class:`aiospamc.client.Client`
        instance.

    :return:
        A successful response with a "Spam" header showing if the message is
        considered spam as well as the score.  The body contains the modified
        message headers, but not the content of the message.

    :raises BadResponse: If the response from SPAMD is ill-formed this exception will be raised.
    :raises AIOSpamcConnectionFailed: Raised if an error occurred when trying to connect.
    :raises UsageException: Error in command line usage.
    :raises DataErrorException: Error with data format.
    :raises NoInputException: Cannot open input.
    :raises NoUserException: Addressee unknown.
    :raises NoHostException: Hostname unknown.
    :raises UnavailableException: Service unavailable.
    :raises InternalSoftwareException: Internal software error.
    :raises OSErrorException: System error.
    :raises OSFileException: Operating system file missing.
    :raises CantCreateException: Cannot create output file.
    :raises IOErrorException: Input/output error.
    :raises TemporaryFailureException: Temporary failure, may reattempt.
    :raises ProtocolException: Error in the protocol.
    :raises NoPermissionException: Permission denied.
    :raises ConfigException: Error in configuration.
    :raises TimeoutException: Timeout during connection.
    '''

    c = Client(host=host, port=port, **kwargs, loop=loop)
    response = await c.headers(message)

    return response


async def ping(
        *,
        host: str = 'Localhost',
        port: int = 783,
        loop: asyncio.AbstractEventLoop = None,
        **kwargs) -> Response:
    '''Sends a ping to the SPAMD service.

    :param host: Hostname or IP address of the SPAMD service, defaults to localhost.
    :param port: Port number for the SPAMD service, defaults to 783.
    :param loop: The asyncio event loop.
    :param kwargs:
        Additional options to pass to the :class:`aiospamc.client.Client` instance.

    :return: A response with "PONG".

    :raises BadResponse: If the response from SPAMD is ill-formed this exception will be raised.
    :raises AIOSpamcConnectionFailed: Raised if an error occurred when trying to connect.
    :raises UsageException: Error in command line usage.
    :raises DataErrorException: Error with data format.
    :raises NoInputException: Cannot open input.
    :raises NoUserException: Addressee unknown.
    :raises NoHostException: Hostname unknown.
    :raises UnavailableException: Service unavailable.
    :raises InternalSoftwareException: Internal software error.
    :raises OSErrorException: System error.
    :raises OSFileException: Operating system file missing.
    :raises CantCreateException: Cannot create output file.
    :raises IOErrorException: Input/output error.
    :raises TemporaryFailureException: Temporary failure, may reattempt.
    :raises ProtocolException: Error in the protocol.
    :raises NoPermissionException: Permission denied.
    :raises ConfigException: Error in configuration.
    :raises TimeoutException: Timeout during connection.
    '''

    c = Client(host=host, port=port, **kwargs, loop=loop)
    response = await c.ping()

    return response


async def process(
        message: Union[bytes, SupportsBytes],
        *,
        host: str = 'Localhost',
        port: int = 783,
        loop: asyncio.AbstractEventLoop = None,
        **kwargs) -> Response:
    '''Checks a message if it's spam and return a response with a score header.

    :param message: Copy of the message.
    :param host: Hostname or IP address of the SPAMD service, defaults to localhost.
    :param port: Port number for the SPAMD service, defaults to 783.
    :param loop: The asyncio event loop.
    :param kwargs:
        Additional options to pass to the :class:`aiospamc.client.Client` instance.

    :return:
        A successful response with a "Spam" header showing if the message is
        considered spam as well as the score.  The body contains a modified
        copy of the message.

    :raises BadResponse: If the response from SPAMD is ill-formed this exception will be raised.
    :raises AIOSpamcConnectionFailed: Raised if an error occurred when trying to connect.
    :raises UsageException: Error in command line usage.
    :raises DataErrorException: Error with data format.
    :raises NoInputException: Cannot open input.
    :raises NoUserException: Addressee unknown.
    :raises NoHostException: Hostname unknown.
    :raises UnavailableException: Service unavailable.
    :raises InternalSoftwareException: Internal software error.
    :raises OSErrorException: System error.
    :raises OSFileException: Operating system file missing.
    :raises CantCreateException: Cannot create output file.
    :raises IOErrorException: Input/output error.
    :raises TemporaryFailureException: Temporary failure, may reattempt.
    :raises ProtocolException: Error in the protocol.
    :raises NoPermissionException: Permission denied.
    :raises ConfigException: Error in configuration.
    :raises TimeoutException: Timeout during connection.
    '''

    c = Client(host=host, port=port, **kwargs, loop=loop)
    response = await c.process(message)

    return response


async def report(
        message: Union[bytes, SupportsBytes],
        *,
        host: str = 'Localhost',
        port: int = 783,
        loop: asyncio.AbstractEventLoop = None,
        **kwargs) -> Response:
    '''Checks a message if it's spam and return a response with a score header.

    :param message: Copy of the message.
    :param host: Hostname or IP address of the SPAMD service, defaults to localhost.
    :param port: Port number for the SPAMD service, defaults to 783.
    :param loop: The asyncio event loop.
    :param kwargs:
        Additional options to pass to the :class:`aiospamc.client.Client` instance.

    :return:
        A successful response with a "Spam" header showing if the message is
        considered spam as well as the score.  The body contains a report.

    :raises BadResponse: If the response from SPAMD is ill-formed this exception will be raised.
    :raises AIOSpamcConnectionFailed: Raised if an error occurred when trying to connect.
    :raises UsageException: Error in command line usage.
    :raises DataErrorException: Error with data format.
    :raises NoInputException: Cannot open input.
    :raises NoUserException: Addressee unknown.
    :raises NoHostException: Hostname unknown.
    :raises UnavailableException: Service unavailable.
    :raises InternalSoftwareException: Internal software error.
    :raises OSErrorException: System error.
    :raises OSFileException: Operating system file missing.
    :raises CantCreateException: Cannot create output file.
    :raises IOErrorException: Input/output error.
    :raises TemporaryFailureException: Temporary failure, may reattempt.
    :raises ProtocolException: Error in the protocol.
    :raises NoPermissionException: Permission denied.
    :raises ConfigException: Error in configuration.
    :raises TimeoutException: Timeout during connection.
    '''

    c = Client(host=host, port=port, **kwargs, loop=loop)
    response = await c.report(message)

    return response


async def report_if_spam(
        message: Union[bytes, SupportsBytes],
        *,
        host: str = 'Localhost',
        port: int = 783,
        loop: asyncio.AbstractEventLoop = None,
        **kwargs) -> Response:
    '''Checks a message if it's spam and return a response with a score header.

    :param message: Copy of the message.
    :param host: Hostname or IP address of the SPAMD service, defaults to localhost.
    :param port: Port number for the SPAMD service, defaults to 783.
    :param loop: The asyncio event loop.
    :param kwargs:
        Additional options to pass to the :class:`aiospamc.client.Client` instance.

    :return:
        A successful response with a "Spam" header showing if the message is
        considered spam as well as the score.  The body contains a report if
        the message is considered spam.

    :raises BadResponse: If the response from SPAMD is ill-formed this exception will be raised.
    :raises AIOSpamcConnectionFailed: Raised if an error occurred when trying to connect.
    :raises UsageException: Error in command line usage.
    :raises DataErrorException: Error with data format.
    :raises NoInputException: Cannot open input.
    :raises NoUserException: Addressee unknown.
    :raises NoHostException: Hostname unknown.
    :raises UnavailableException: Service unavailable.
    :raises InternalSoftwareException: Internal software error.
    :raises OSErrorException: System error.
    :raises OSFileException: Operating system file missing.
    :raises CantCreateException: Cannot create output file.
    :raises IOErrorException: Input/output error.
    :raises TemporaryFailureException: Temporary failure, may reattempt.
    :raises ProtocolException: Error in the protocol.
    :raises NoPermissionException: Permission denied.
    :raises ConfigException: Error in configuration.
    :raises TimeoutException: Timeout during connection.
    '''

    c = Client(host=host, port=port, **kwargs, loop=loop)
    response = await c.report_if_spam(message)

    return response


async def symbols(message: Union[bytes, SupportsBytes],
                  *,
                  host: str = 'Localhost',
                  port: int = 783,
                  loop: asyncio.AbstractEventLoop = None,
                  **kwargs) -> Response:
    '''Checks a message if it's spam and return a response with a score header.

    :param message: Copy of the message.
    :param host: Hostname or IP address of the SPAMD service, defaults to localhost.
    :param port: Port number for the SPAMD service, defaults to 783.
    :param loop: The asyncio event loop.
    :param kwargs:
        Additional options to pass to the :class:`aiospamc.client.Client` instance.

    :return:
        A successful response with a "Spam" header showing if the message is
        considered spam as well as the score.  The body contains a
        comma-separated list of the symbols that were hit.

    :raises BadResponse: If the response from SPAMD is ill-formed this exception will be raised.
    :raises AIOSpamcConnectionFailed: Raised if an error occurred when trying to connect.
    :raises UsageException: Error in command line usage.
    :raises DataErrorException: Error with data format.
    :raises NoInputException: Cannot open input.
    :raises NoUserException: Addressee unknown.
    :raises NoHostException: Hostname unknown.
    :raises UnavailableException: Service unavailable.
    :raises InternalSoftwareException: Internal software error.
    :raises OSErrorException: System error.
    :raises OSFileException: Operating system file missing.
    :raises CantCreateException: Cannot create output file.
    :raises IOErrorException: Input/output error.
    :raises TemporaryFailureException: Temporary failure, may reattempt.
    :raises ProtocolException: Error in the protocol.
    :raises NoPermissionException: Permission denied.
    :raises ConfigException: Error in configuration.
    :raises TimeoutException: Timeout during connection.
    '''

    c = Client(host=host, port=port, **kwargs, loop=loop)
    response = await c.symbols(message)

    return response


async def tell(
        message: Union[bytes, SupportsBytes],
        message_class: Union[str, MessageClassOption],
        *,
        host: str = 'Localhost',
        port: int = 783,
        remove_action: Union[str, ActionOption] = None,
        set_action: Union[str, ActionOption] = None,
        loop: asyncio.AbstractEventLoop = None, **kwargs) -> Response:
    '''Checks a message if it's spam and return a response with a score header.

    :param message: Copy of the message.
    :param message_class: How to classify the message, either "ham" or "spam".
    :param host: Hostname or IP address of the SPAMD service, defaults to localhost.
    :param port: Port number for the SPAMD service, defaults to 783.
    :param loop: The asyncio event loop.
    :param kwargs:
        Additional options to pass to the :class:`aiospamc.client.Client` instance.

    :return:
        A successful response with "DidSet" and/or "DidRemove" headers along with the
        actions that were taken.

    :raises BadResponse: If the response from SPAMD is ill-formed this exception will be raised.
    :raises AIOSpamcConnectionFailed: Raised if an error occurred when trying to connect.
    :raises UsageException: Error in command line usage.
    :raises DataErrorException: Error with data format.
    :raises NoInputException: Cannot open input.
    :raises NoUserException: Addressee unknown.
    :raises NoHostException: Hostname unknown.
    :raises UnavailableException: Service unavailable.
    :raises InternalSoftwareException: Internal software error.
    :raises OSErrorException: System error.
    :raises OSFileException: Operating system file missing.
    :raises CantCreateException: Cannot create output file.
    :raises IOErrorException: Input/output error.
    :raises TemporaryFailureException: Temporary failure, may reattempt.
    :raises ProtocolException: Error in the protocol.
    :raises NoPermissionException: Permission denied.
    :raises ConfigException: Error in configuration.
    :raises TimeoutException: Timeout during connection.
    '''

    c = Client(host=host, port=port, **kwargs, loop=loop)
    response = await c.tell(
        message=message,
        message_class=message_class,
        remove_action=remove_action,
        set_action=set_action
    )

    return response



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['aiospamc', 'aiospamc.connections']

package_data = \
{'': ['*']}

install_requires = \
['certifi>=2019.9,<2020.0']

setup_kwargs = {
    'name': 'aiospamc',
    'version': '0.6.0',
    'description': "An asyncio-based library to communicate with SpamAssassin's SPAMD service.",
    'long_description': "========\naiospamc\n========\n\n.. image:: https://img.shields.io/azure-devops/build/mjcaley/aiospamc/2/master\n    :target: https://dev.azure.com/mjcaley/aiospamc/_build\n.. image:: https://img.shields.io/azure-devops/coverage/mjcaley/aiospamc/2/master\n    :target: https://dev.azure.com/mjcaley/aiospamc/_build\n.. image:: https://img.shields.io/readthedocs/aiospamc\n    :target: https://aiospamc.readthedocs.io/en/latest/\n.. image:: https://img.shields.io/pypi/v/aiospamc\n    :target: https://pypi.org/project/aiospamc/\n\n-----------\nDescription\n-----------\n\nPython asyncio-based library that implements the SPAMC/SPAMD client protocol used by SpamAssassin.\n\n-------------\nDocumentation\n-------------\n\nDocumentation can be found at: https://aiospamc.readthedocs.io/\n\n------------\nRequirements\n------------\n\n* Python 3.5 or higher\n\n-------\nExample\n-------\n\n.. code:: python\n    \n    import asyncio\n    import aiospamc\n\n\n    GTUBE = '''Subject: Test spam mail (GTUBE)\n    Message-ID: <GTUBE1.1010101@example.net>\n    Date: Wed, 23 Jul 2003 23:30:00 +0200\n    From: Sender <sender@example.net>\n    To: Recipient <recipient@example.net>\n    Precedence: junk\n    MIME-Version: 1.0\n    Content-Type: text/plain; charset=us-ascii\n    Content-Transfer-Encoding: 7bit\n\n    This is the GTUBE, the\n        Generic\n        Test for\n        Unsolicited\n        Bulk\n        Email\n\n    If your spam filter supports it, the GTUBE provides a test by which you\n    can verify that the filter is installed correctly and is detecting incoming\n    spam. You can send yourself a test mail containing the following string of\n    characters (in upper case and with no white spaces and line breaks):\n\n    XJS*C4JDBQADN1.NSBN3*2IDNEN*GTUBE-STANDARD-ANTI-UBE-TEST-EMAIL*C.34X\n\n    You should send this test mail from an account outside of your network.\n    '''.encode('ascii')\n\n    loop = asyncio.get_event_loop()\n    responses = loop.run_until_complete(asyncio.gather(\n\n        aiospamc.ping(host='localhost', loop=loop),\n        aiospamc.check(GTUBE, host='localhost', loop=loop),\n        aiospamc.headers(GTUBE, host='localhost', loop=loop)\n\n    ))\n    print(responses)\n",
    'author': 'Michael Caley',
    'author_email': 'mjcaley@darkarctic.com',
    'url': 'https://github.com/mjcaley/aiospamc',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3

'''TCP socket connection and manager.'''

import asyncio
from ssl import SSLContext
from typing import Tuple

from . import Connection
from . import ConnectionManager
from ..exceptions import AIOSpamcConnectionFailed


class TcpConnectionManager(ConnectionManager):
    '''Creates new connections based on host and port provided.'''

    def __init__(self, host: str, port: int, ssl: SSLContext = None, loop: asyncio.AbstractEventLoop = None) -> None:
        '''Constructor for TcpConnectionManager.

        :param host: Hostname or IP address of server.
        :param port: Port number
        :param ssl: SSL/TLS context.
        :param loop: The asyncio event loop.
        '''

        self.host = host
        self.port = port
        self.ssl = ssl
        super().__init__(loop)

    def __repr__(self) -> str:
        return '{}(host={}, port={}, ssl={})'.format(self.__class__.__name__,
                                                     repr(self.host),
                                                     repr(self.port),
                                                     self.ssl)

    def new_connection(self) -> 'TcpConnection':
        '''Creates a new TCP connection.

        :raises AIOSpamcConnectionFailed:
        '''

        return TcpConnection(self.host, self.port, self.ssl, self.loop)


class TcpConnection(Connection):
    '''Manages a TCP connection.'''

    def __init__(self, host: str, port: int, ssl: SSLContext = None, loop: asyncio.AbstractEventLoop = None):
        '''Constructor for TcpConnection.

        :param host: Hostname or IP address of server.
        :param port: Port number
        :param ssl: SSL/TLS context.
        '''

        self.host = host
        self.port = port
        self.ssl = ssl
        super().__init__(loop)

    def __repr__(self) -> str:
        return '{}(host={}, port={}, ssl={})'.format(self.__class__.__name__,
                                                     repr(self.host),
                                                     repr(self.port),
                                                     self.ssl)

    async def open(self) -> Tuple[asyncio.StreamReader, asyncio.StreamWriter]:
        '''Opens a connection.

        :raises AIOSpamcConnectionFailed:
        '''

        try:

            reader, writer = await asyncio.open_connection(self.host,
                                                           self.port,
                                                           ssl=self.ssl,
                                                           loop=self.loop)
        except (ConnectionRefusedError, OSError) as error:
            raised = AIOSpamcConnectionFailed(error)
            self.logger.exception('Exception occurred when connecting to %s:%s: %s',
                                  self.host,
                                  self.port,
                                  raised)
            raise raised

        return reader, writer

    @property
    def connection_string(self) -> str:
        '''String representation of the connection.'''

        return ':'.join([self.host, str(self.port)])
